/**************************************************************************
*
*	CommsClient.h
*
*	Contains the comms client interface
*
*	Author: Saxon Druce
*
*	Copyright (c) 2004 Block Software Pty Ltd
*
**************************************************************************/

#ifndef _COMMSCLIENT_H
#define _COMMSCLIENT_H

/*************************************************************************/
// Include files
/*************************************************************************/

#include <string>

#include "CommsMessage.h"

/*************************************************************************/
// Defines
/*************************************************************************/

/*************************************************************************/
// Public types
/*************************************************************************/

// CommsClient_t class - implements a client of the communications system.
// Specified clients can be implemented via either inheritance or delegation
// of this class.
class CommsClient_t
{
public:

	// Constructor. Takes the name which is used to identify this client
	// to the server when it tries to connect, and the version number
	// reported to the server so the server can decide if it is compatible 
	// with this client.
	CommsClient_t(
		const std::string& Name,
		int Version);

	// Destructor
	virtual ~CommsClient_t();

	// Connect() - Connects to the specified port on the specified host
	// (name or IP address). Throws an exception on failure to connect.
	void Connect(
		const std::string& HostName,
		int Port);

	// SendMessage() - Sends the given message to the server, along with
	// the given message ID. Blocks until the server sends back a response,
	// which is returned.
	CommsMessage_t SendMessage(int MessageID, const CommsMessage_t& Message);

private:

	// private data
	class Data_t;
	Data_t *m_Data;
};

/*************************************************************************/

#endif // _COMMSCLIENT_H
