/**************************************************************************
*
*	ControlForm.cs
*
*	Control form for interactive robot sample programme
*
*	Author: Saxon Druce
*
*	Copyright (c) 2004 Block Software Pty Ltd
*
**************************************************************************/

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

using ControllerNet;

namespace InteractiveRobot
{
	/// <summary>
	/// Summary description for ControlForm.
	/// </summary>
	public class ControlForm : System.Windows.Forms.Form
	{
		private System.ComponentModel.IContainer components;

		/// <summary>
		/// Connection to robot API
		/// </summary>
		Controller m_controller;

		/// <summary>
		/// Whether the up/down/left/right keys are currently pressed
		/// </summary>
		bool m_keyUpDown = false;
		bool m_keyDownDown = false;
		bool m_keyLeftDown = false;
		bool m_keyRightDown = false;

		/// <summary>
		/// Controls
		/// </summary>
		private System.Windows.Forms.Timer m_timer;
		private System.Windows.Forms.Label m_labelInstructionsUpKey;
		private System.Windows.Forms.Label m_labelInstructionsUpMeaning;
		private System.Windows.Forms.Label m_labelInstructionsDownMeaning;
		private System.Windows.Forms.Label m_labelInstructionsDownKey;
		private System.Windows.Forms.Label m_labelInstructionsLeftKey;
		private System.Windows.Forms.Label m_labelInstructionsLeftMeaning;
		private System.Windows.Forms.Label m_labelInstructionsRightKey;
		private System.Windows.Forms.Label m_labelInstructionsRightMeaning;
		private System.Windows.Forms.Label m_labelInstructionsSpaceKey;
		private System.Windows.Forms.Label m_labelInstructionsSpaceMeaning;
		private System.Windows.Forms.Label m_labelInstructionsEscapeKey;
		private System.Windows.Forms.Label m_labelInstructionsEscapeMeaning;

		/// <summary>
		/// Constructor
		/// </summary>
		public ControlForm(
			string hostName, int port,
			string name, string owner)
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			// note that if an exception occurs in the following
			// (eg unable to connect), this is caught by the main
			// form, with the end result that this control form
			// is never made visible

			// connect
			m_controller = new Controller();
			m_controller.Connect(hostName, port);

			// create robot
			int robotID = m_controller.CreateRobot();

			// set name and owner
			m_controller.SetName(name);
			m_controller.SetOwner(owner);

			// set colour
			m_controller.SetColour(255, 128, 0);

			// buy equipment
			m_controller.PurchaseEngine(4, 90);
			m_controller.PurchaseSensor(10, 60, 40);
			m_controller.PurchaseBattery(50);
			m_controller.PurchaseTurret(0, 0);
			m_controller.PurchaseWeapon(2, 4, 2, 3);
			m_controller.PurchaseSolarPanel(1);

			// enter simulation
			m_controller.EnterSimulation();

			// enable timer now that we've successfully started
			m_timer.Enabled = true;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			this.m_timer = new System.Windows.Forms.Timer(this.components);
			this.m_labelInstructionsUpKey = new System.Windows.Forms.Label();
			this.m_labelInstructionsUpMeaning = new System.Windows.Forms.Label();
			this.m_labelInstructionsDownMeaning = new System.Windows.Forms.Label();
			this.m_labelInstructionsDownKey = new System.Windows.Forms.Label();
			this.m_labelInstructionsLeftKey = new System.Windows.Forms.Label();
			this.m_labelInstructionsLeftMeaning = new System.Windows.Forms.Label();
			this.m_labelInstructionsRightKey = new System.Windows.Forms.Label();
			this.m_labelInstructionsRightMeaning = new System.Windows.Forms.Label();
			this.m_labelInstructionsSpaceKey = new System.Windows.Forms.Label();
			this.m_labelInstructionsSpaceMeaning = new System.Windows.Forms.Label();
			this.m_labelInstructionsEscapeKey = new System.Windows.Forms.Label();
			this.m_labelInstructionsEscapeMeaning = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// m_timer
			// 
			this.m_timer.Tick += new System.EventHandler(this.m_timer_Tick);
			// 
			// m_labelInstructionsUpKey
			// 
			this.m_labelInstructionsUpKey.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.m_labelInstructionsUpKey.Location = new System.Drawing.Point(8, 8);
			this.m_labelInstructionsUpKey.Name = "m_labelInstructionsUpKey";
			this.m_labelInstructionsUpKey.Size = new System.Drawing.Size(72, 16);
			this.m_labelInstructionsUpKey.TabIndex = 0;
			this.m_labelInstructionsUpKey.Text = "Up arrow";
			// 
			// m_labelInstructionsUpMeaning
			// 
			this.m_labelInstructionsUpMeaning.Location = new System.Drawing.Point(96, 8);
			this.m_labelInstructionsUpMeaning.Name = "m_labelInstructionsUpMeaning";
			this.m_labelInstructionsUpMeaning.Size = new System.Drawing.Size(96, 16);
			this.m_labelInstructionsUpMeaning.TabIndex = 0;
			this.m_labelInstructionsUpMeaning.Text = "Move forwards";
			// 
			// m_labelInstructionsDownMeaning
			// 
			this.m_labelInstructionsDownMeaning.Location = new System.Drawing.Point(96, 24);
			this.m_labelInstructionsDownMeaning.Name = "m_labelInstructionsDownMeaning";
			this.m_labelInstructionsDownMeaning.Size = new System.Drawing.Size(96, 16);
			this.m_labelInstructionsDownMeaning.TabIndex = 0;
			this.m_labelInstructionsDownMeaning.Text = "Move backwards";
			// 
			// m_labelInstructionsDownKey
			// 
			this.m_labelInstructionsDownKey.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.m_labelInstructionsDownKey.Location = new System.Drawing.Point(8, 24);
			this.m_labelInstructionsDownKey.Name = "m_labelInstructionsDownKey";
			this.m_labelInstructionsDownKey.Size = new System.Drawing.Size(72, 16);
			this.m_labelInstructionsDownKey.TabIndex = 0;
			this.m_labelInstructionsDownKey.Text = "Down arrow";
			// 
			// m_labelInstructionsLeftKey
			// 
			this.m_labelInstructionsLeftKey.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.m_labelInstructionsLeftKey.Location = new System.Drawing.Point(8, 40);
			this.m_labelInstructionsLeftKey.Name = "m_labelInstructionsLeftKey";
			this.m_labelInstructionsLeftKey.Size = new System.Drawing.Size(72, 16);
			this.m_labelInstructionsLeftKey.TabIndex = 0;
			this.m_labelInstructionsLeftKey.Text = "Left arrow";
			// 
			// m_labelInstructionsLeftMeaning
			// 
			this.m_labelInstructionsLeftMeaning.Location = new System.Drawing.Point(96, 40);
			this.m_labelInstructionsLeftMeaning.Name = "m_labelInstructionsLeftMeaning";
			this.m_labelInstructionsLeftMeaning.Size = new System.Drawing.Size(96, 16);
			this.m_labelInstructionsLeftMeaning.TabIndex = 0;
			this.m_labelInstructionsLeftMeaning.Text = "Turn left";
			// 
			// m_labelInstructionsRightKey
			// 
			this.m_labelInstructionsRightKey.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.m_labelInstructionsRightKey.Location = new System.Drawing.Point(8, 56);
			this.m_labelInstructionsRightKey.Name = "m_labelInstructionsRightKey";
			this.m_labelInstructionsRightKey.Size = new System.Drawing.Size(72, 16);
			this.m_labelInstructionsRightKey.TabIndex = 0;
			this.m_labelInstructionsRightKey.Text = "Right arrow";
			// 
			// m_labelInstructionsRightMeaning
			// 
			this.m_labelInstructionsRightMeaning.Location = new System.Drawing.Point(96, 56);
			this.m_labelInstructionsRightMeaning.Name = "m_labelInstructionsRightMeaning";
			this.m_labelInstructionsRightMeaning.Size = new System.Drawing.Size(96, 16);
			this.m_labelInstructionsRightMeaning.TabIndex = 0;
			this.m_labelInstructionsRightMeaning.Text = "Turn right";
			// 
			// m_labelInstructionsSpaceKey
			// 
			this.m_labelInstructionsSpaceKey.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.m_labelInstructionsSpaceKey.Location = new System.Drawing.Point(8, 72);
			this.m_labelInstructionsSpaceKey.Name = "m_labelInstructionsSpaceKey";
			this.m_labelInstructionsSpaceKey.Size = new System.Drawing.Size(72, 16);
			this.m_labelInstructionsSpaceKey.TabIndex = 0;
			this.m_labelInstructionsSpaceKey.Text = "Space";
			// 
			// m_labelInstructionsSpaceMeaning
			// 
			this.m_labelInstructionsSpaceMeaning.Location = new System.Drawing.Point(96, 72);
			this.m_labelInstructionsSpaceMeaning.Name = "m_labelInstructionsSpaceMeaning";
			this.m_labelInstructionsSpaceMeaning.Size = new System.Drawing.Size(96, 16);
			this.m_labelInstructionsSpaceMeaning.TabIndex = 0;
			this.m_labelInstructionsSpaceMeaning.Text = "Fire weapon";
			// 
			// m_labelInstructionsEscapeKey
			// 
			this.m_labelInstructionsEscapeKey.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.m_labelInstructionsEscapeKey.Location = new System.Drawing.Point(8, 88);
			this.m_labelInstructionsEscapeKey.Name = "m_labelInstructionsEscapeKey";
			this.m_labelInstructionsEscapeKey.Size = new System.Drawing.Size(72, 16);
			this.m_labelInstructionsEscapeKey.TabIndex = 0;
			this.m_labelInstructionsEscapeKey.Text = "Escape";
			// 
			// m_labelInstructionsEscapeMeaning
			// 
			this.m_labelInstructionsEscapeMeaning.Location = new System.Drawing.Point(96, 88);
			this.m_labelInstructionsEscapeMeaning.Name = "m_labelInstructionsEscapeMeaning";
			this.m_labelInstructionsEscapeMeaning.Size = new System.Drawing.Size(96, 16);
			this.m_labelInstructionsEscapeMeaning.TabIndex = 0;
			this.m_labelInstructionsEscapeMeaning.Text = "Exit";
			// 
			// ControlForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(208, 110);
			this.Controls.Add(this.m_labelInstructionsUpKey);
			this.Controls.Add(this.m_labelInstructionsUpMeaning);
			this.Controls.Add(this.m_labelInstructionsDownMeaning);
			this.Controls.Add(this.m_labelInstructionsDownKey);
			this.Controls.Add(this.m_labelInstructionsLeftKey);
			this.Controls.Add(this.m_labelInstructionsLeftMeaning);
			this.Controls.Add(this.m_labelInstructionsRightKey);
			this.Controls.Add(this.m_labelInstructionsRightMeaning);
			this.Controls.Add(this.m_labelInstructionsSpaceKey);
			this.Controls.Add(this.m_labelInstructionsSpaceMeaning);
			this.Controls.Add(this.m_labelInstructionsEscapeKey);
			this.Controls.Add(this.m_labelInstructionsEscapeMeaning);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
			this.Name = "ControlForm";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
			this.Text = "Interactive Robot";
			this.KeyDown += new System.Windows.Forms.KeyEventHandler(this.ControlForm_KeyDown);
			this.KeyUp += new System.Windows.Forms.KeyEventHandler(this.ControlForm_KeyUp);
			this.Closed += new System.EventHandler(this.ControlForm_Closed);
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// Called when a key is pressed
		/// </summary>
		private void ControlForm_KeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			try
			{
				switch (e.KeyCode)
				{
					// update whether the driving keys are down
					case Keys.Up:
						m_keyUpDown = true;
						break;
					case Keys.Down:
						m_keyDownDown = true;
						break;
					case Keys.Left:
						m_keyLeftDown = true;
						break;
					case Keys.Right:
						m_keyRightDown = true;
						break;

					case Keys.Space:
						// space pressed, so fire
						m_controller.FireWeapon();
						break;

					case Keys.Escape:
						// escape pressed, so close form
						this.Close();
						break;
				}
			}
			catch (Exception ex)
			{
				m_timer.Enabled = false;

				MessageBox.Show(
					ex.Message, 
					"Error", 
					MessageBoxButtons.OK, 
					MessageBoxIcon.Error);

				this.Close();
			}
		}

		/// <summary>
		/// Called when a key is released
		/// </summary>
		private void ControlForm_KeyUp(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			switch (e.KeyCode)
			{
				// update whether the driving keys are up
				case Keys.Up:
					m_keyUpDown = false;
					break;
				case Keys.Down:
					m_keyDownDown = false;
					break;
				case Keys.Left:
					m_keyLeftDown = false;
					break;
				case Keys.Right:
					m_keyRightDown = false;
					break;
			}
		}

		/// <summary>
		/// Called when the timer ticks
		/// </summary>
		private void m_timer_Tick(object sender, System.EventArgs e)
		{
			try
			{
				// work out desired throttle
				double throttle = 0;
				if (m_keyUpDown)
					throttle += 100;
				if (m_keyDownDown)
					throttle -= 100;

				// then set it
				m_controller.SetThrottle(throttle);

				// work out desired steering angle
				double steeringAngle = 0;
				if (m_keyLeftDown)
					steeringAngle -= Math.PI / 2;
				if (m_keyRightDown)
					steeringAngle += Math.PI / 2;

				// set desired orientation to current orientation 
				// plus steering angle
				m_controller.SetDesiredOrientation(
					m_controller.GetOrientation() + steeringAngle);
			}
			catch (Exception ex)
			{
				m_timer.Enabled = false;

				MessageBox.Show(
					ex.Message, 
					"Error", 
					MessageBoxButtons.OK, 
					MessageBoxIcon.Error);

				this.Close();
			}
		}

		/// <summary>
		/// Called when the form is closed
		/// </summary>
		private void ControlForm_Closed(object sender, System.EventArgs e)
		{
			// disconnect (if connected)
			if (m_controller != null)
			{
				m_controller.Dispose();
				m_controller = null;
			}
		}
	}
}
